﻿using paysafe;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml;

public class Payout
{
    private Dictionary<string, string> param = new Dictionary<string, string>();
    private String url;
    private String response;
    private String errorC;
    private String resultC;
    private String errorCD;

    public Payout(String username, String password, bool productive)
    {
        param.Add("username", username);
        param.Add("password", password);
        url = productive ? "https://soa.paysafecard.com/psc/services/PscService?wsdl" : "https://soatest.paysafecard.com/psc/services/PscService?wsdl";
    }

    public void SetTransaction(String amount, String currency, String ptid, String utcOffset)
    {
        param.Add("ptid", String.IsNullOrEmpty(ptid) ? Generateptid() : ptid);
        param.Add("amount", amount.Replace(",", "."));
        param.Add("currency", currency.ToUpper());
        param.Add("utcOffset", utcOffset);
    }

    public void SetCustomer(String merchantClientId, String customerId, String customerIdType)
    {
        param.Add("customerId", customerId);
        param.Add("merchantClientId", merchantClientId);
        param.Add("customerIdType", customerIdType);
    }

    public void SetCustomerDetails(String firstName, String lastName, String dateOfBirth)
    {
        param.Add("firstName", firstName);
        param.Add("lastName", lastName);
        param.Add("dateOfBirth", dateOfBirth);
    }

    public void PayoutDo(bool validationOnly)
    {
        try
        {
            param.Add("validationOnly", validationOnly.ToString().ToLower());
            PSCSOAPClient connection = new PSCSOAPClient();
            connection.Addelement(param);
            response = connection.Soapcall(url);

            InitResultVars(response);
        }
        catch (Exception ex)
        {
            Console.WriteLine(ex);
        }
    }

    public Dictionary<string, string> GetPayoutState()
    {
        Dictionary<string, string> responsecodes = new Dictionary<string, string>();
        try
        {
            PSCSOAPClient connection = new PSCSOAPClient();
            connection.AddelementState(param);
            response = connection.Soapcall(url);

            InitResultVars(response);

            XmlDocument xm = new XmlDocument();
            xm.LoadXml(response);
            XmlNamespaceManager manager = new XmlNamespaceManager(xm.NameTable);
            manager.AddNamespace("ns1", "urn:pscservice");
            XmlNode errorCode = xm.SelectSingleNode("//ns1:errorCode", manager);
            XmlNode resultCode = xm.SelectSingleNode("//ns1:resultCode", manager);
            XmlNode errorCodeDescription = xm.SelectSingleNode("//ns1:errorCodeDescription", manager);
            XmlNode totalPayoutAmount = xm.SelectSingleNode("//ns1:totalPayoutAmount", manager);
            XmlNode totalPaymentAmount = xm.SelectSingleNode("//ns1:totalPaymentAmount", manager);
            XmlNode creditLine = xm.SelectSingleNode("//ns1:creditLine", manager);
            XmlNode totalPayoutBalance = xm.SelectSingleNode("//ns1:totalPayoutBalance", manager);
            XmlNode dailyPayoutLimit = xm.SelectSingleNode("//ns1:dailyPayoutLimit", manager);
            XmlNode dailyPayoutAmount = xm.SelectSingleNode("//ns1:dailyPayoutAmount", manager);
            XmlNode dailyPayoutBalance = xm.SelectSingleNode("//ns1:dailyPayoutBalance", manager);
            responsecodes.Add("errorCode", errorCode.InnerText);
            responsecodes.Add("resultCode", resultCode.InnerText);
            responsecodes.Add("errorCodeDescription", errorCodeDescription.InnerText);
            responsecodes.Add("totalPayoutAmount", totalPayoutAmount.InnerText);
            responsecodes.Add("totalPaymentAmount", totalPaymentAmount.InnerText);
            responsecodes.Add("totalPayoutBalance", totalPayoutBalance.InnerText);
            responsecodes.Add("dailyPayoutLimit", dailyPayoutLimit.InnerText);
            responsecodes.Add("dailyPayoutAmount", dailyPayoutAmount.InnerText);
            responsecodes.Add("dailyPayoutBalance", dailyPayoutBalance.InnerText);
        }
        catch (Exception ex)
        {
            Console.WriteLine(ex);
        }
        return responsecodes;
    }

    public String GetSoapResponse()
    {
        return response.ToString();
    }

    public String GetResultCode()
    {
        return resultC;
    }

    public String GetErrorCode()
    {
        return errorC;
    }

    public String GetErrorCodeDescription()
    {
        return errorCD;
    }
    public String getError(String resultcode)
    {
        switch (resultcode)
        {
            case "3167_20":
            case "3167_40":
            case "3167":
            case "3170_20":
            case "3170_40":
            case "3170":
            case "3194_20":
            case "3194_40":
            case "3194":
            case "3197":
            case "3198":
                return "Unfortunately there has been a problem executing your request. Please check your e-mail inbox for further information from paysafecard on how to solve the issue and try again.";
            case "3162":
                return "Unfortunately, no my paysafecard account exists under the e-mail address you have entered. Please check the address for a typing error. If you do not have a my paysafecard account, you can <a href=\"https://mypins.paysafecard.com/customer-registration/StartRegistration.xhtml\" id=\"paysafecard_link\">register for one online now for free</a>.";
            case "3195_20":
            case "3195_40":
            case "3195":
                return "The personal details associated with your my paysafecard account do not match the details of this account. Please check the first names, surnames and dates of birth entered in both accounts and request the payout again.";
            default:
                return "Sorry, the payout was not completed successfully! There was no money transfered. Please contact our customer support: support@company.com";
        }
    }

    private String Generateptid()
    {
        Random rand = new Random();
        int randomNum = rand.Next(1, 9999);

        DateTime time = DateTime.Now;
        string format = "yyyymmdd-Hms";
        String date = time.ToString(format);

        String rtext = "test_" + date + "_" + randomNum;

        return rtext;
    }

    private void InitResultVars(String response)
    {
        XmlDocument xm = new XmlDocument();
        xm.LoadXml(response);
        XmlNamespaceManager manager = new XmlNamespaceManager(xm.NameTable);
        manager.AddNamespace("ns1", "urn:pscservice");
        XmlNode errorCode = xm.SelectSingleNode("//ns1:errorCode", manager);
        XmlNode resultCode = xm.SelectSingleNode("//ns1:resultCode", manager);
        XmlNode errorCodeDescription = xm.SelectSingleNode("//ns1:errorCodeDescription", manager);

        errorC = errorCode.InnerText;
        resultC = resultCode.InnerText;
        errorCD = errorCodeDescription.InnerText;
    }
}
